@@ -1,13 +1,17 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
+ * This copy of SIP may also used under the terms of the GNU General Public
+ * License v2 or v3 as published by the Free Software Foundation which can be
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -39,7 +43,7 @@
 
 /* For convenience. */
 
-#define classBaseName(cd)   ((cd)->iff->name->text)
+#define classBaseName(cd)   scopedNameTail((cd)->iff->fqcname)
 #define classFQCName(cd)    ((cd)->iff->fqcname)
 
 
@@ -48,6 +52,7 @@
 #define MOD_HAS_DELAYED_DTORS   0x0001  /* It has a class with a delayed dtor. */
 #define MOD_IS_CONSOLIDATED     0x0002  /* It is a consolidated module. */
 #define MOD_IS_COMPOSITE        0x0004  /* It is a composite module. */
+#define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
@@ -56,6 +61,8 @@
 #define isComposite(m)      ((m)->modflags & MOD_IS_COMPOSITE)
 #define setIsComposite(m)   ((m)->modflags |= MOD_IS_COMPOSITE)
 #define isContainer(m)      ((m)->modflags & (MOD_IS_CONSOLIDATED | MOD_IS_COMPOSITE))
+#define setIsTransformed(m) ((m)->modflags |= MOD_IS_TRANSFORMED)
+#define isTransformed(m)    ((m)->modflags & MOD_IS_TRANSFORMED)
 
 
 /* Handle section flags. */
@@ -78,17 +85,20 @@
 #define CLASS_DTOR_RELEASE_GIL  0x00004000  /* The dtor releases the GIL. */
 #define CLASS_IS_PROTECTED  0x00008000  /* It is protected. */
 #define CLASS_IS_PROTECTED_SAV  0x00010000  /* It is protected (saved). */
-#define CLASS_IS_RENAMED    0x00020000  /* It has a different Python name. */
-#define CLASS_IS_INCOMPLETE 0x00040000  /* The specification is incomplete. */
-#define CLASS_CAN_CREATE    0x00080000  /* It has usable ctors. */
-#define CLASS_IS_EXTERNAL   0x00100000  /* It is external. */
-#define CLASS_IS_DELAYED_DTOR   0x00200000  /* The dtor is delayed. */
-#define CLASS_NO_DEFAULT_CTORS  0x00400000  /* Don't create default ctors. */
-#define CLASS_QOBJECT_SUB   0x00800000  /* It is derived from QObject. */
-#define CLASS_DTOR_HOLD_GIL 0x01000000  /* The dtor holds the GIL. */
-#define CLASS_QT_META_TYPE  0x02000000  /* Register as a Qt meta type. */
-#define CLASS_NO_QMETAOBJECT    0x04000000  /* It has no QMetaObject. */
-#define CLASS_CAN_ASSIGN    0x08000000  /* It has an assignment operator. */
+#define CLASS_IS_INCOMPLETE 0x00020000  /* The specification is incomplete. */
+#define CLASS_CAN_CREATE    0x00040000  /* It has usable ctors. */
+#define CLASS_IS_EXTERNAL   0x00080000  /* It is external. */
+#define CLASS_IS_DELAYED_DTOR   0x00100000  /* The dtor is delayed. */
+#define CLASS_NO_DEFAULT_CTORS  0x00200000  /* Don't create default ctors. */
+#define CLASS_QOBJECT_SUB   0x00400000  /* It is derived from QObject. */
+#define CLASS_DTOR_HOLD_GIL 0x00800000  /* The dtor holds the GIL. */
+#define CLASS_ASSIGN_HELPER 0x01000000  /* Generate an assignment helper. */
+#define CLASS_NO_QMETAOBJECT    0x02000000  /* It has no QMetaObject. */
+#define CLASS_IS_TEMPLATE   0x04000000  /* It is a template class. */
+#define CLASS_IS_DEPRECATED 0x08000000  /* It is deprecated. */
+#define CLASS_CANNOT_COPY   0x10000000  /* It cannot be copied. */
+#define CLASS_CANNOT_ASSIGN 0x20000000  /* It cannot be assigned. */
+#define CLASS_ALLOW_NONE    0x40000000  /* The class will handle None. */
 
 #define hasSigSlots(cd)     ((cd)->classflags & CLASS_HAS_SIGSLOTS)
 #define setHasSigSlots(cd)  ((cd)->classflags |= CLASS_HAS_SIGSLOTS)
@@ -107,10 +117,8 @@
 #define wasProtectedClass(cd)   ((cd)->classflags & CLASS_IS_PROTECTED_SAV)
 #define setWasProtectedClass(cd)    ((cd)->classflags |= CLASS_IS_PROTECTED_SAV)
 #define resetWasProtectedClass(cd)  ((cd)->classflags &= ~CLASS_IS_PROTECTED_SAV)
-#define isReleaseGILDtor(c) ((cd)->classflags & CLASS_DTOR_RELEASE_GIL)
-#define setIsReleaseGILDtor(c)  ((cd)->classflags |= CLASS_DTOR_RELEASE_GIL)
-#define isRenamedClass(cd)  ((cd)->classflags & CLASS_IS_RENAMED)
-#define setIsRenamedClass(cd)   ((cd)->classflags |= CLASS_IS_RENAMED)
+#define isReleaseGILDtor(cd)    ((cd)->classflags & CLASS_DTOR_RELEASE_GIL)
+#define setIsReleaseGILDtor(cd) ((cd)->classflags |= CLASS_DTOR_RELEASE_GIL)
 #define isIncomplete(cd)    ((cd)->classflags & CLASS_IS_INCOMPLETE)
 #define setIsIncomplete(cd) ((cd)->classflags |= CLASS_IS_INCOMPLETE)
 #define canCreate(cd)       ((cd)->classflags & CLASS_CAN_CREATE)
@@ -124,14 +132,23 @@
 #define setNoDefaultCtors(cd)   ((cd)->classflags |= CLASS_NO_DEFAULT_CTORS)
 #define isQObjectSubClass(cd)   ((cd)->classflags & CLASS_QOBJECT_SUB)
 #define setIsQObjectSubClass(cd)    ((cd)->classflags |= CLASS_QOBJECT_SUB)
-#define isHoldGILDtor(c)    ((cd)->classflags & CLASS_DTOR_HOLD_GIL)
-#define setIsHoldGILDtor(c) ((cd)->classflags |= CLASS_DTOR_HOLD_GIL)
-#define registerQtMetaType(c)   ((cd)->classflags & CLASS_QT_META_TYPE)
-#define setRegisterQtMetaType(c)    ((cd)->classflags |= CLASS_QT_META_TYPE)
-#define noQMetaObject(c)    ((cd)->classflags & CLASS_NO_QMETAOBJECT)
-#define setNoQMetaObject(c) ((cd)->classflags |= CLASS_NO_QMETAOBJECT)
-#define canAssign(c)        ((cd)->classflags & CLASS_CAN_ASSIGN)
-#define setCanAssign(c)     ((cd)->classflags |= CLASS_CAN_ASSIGN)
+#define isHoldGILDtor(cd)   ((cd)->classflags & CLASS_DTOR_HOLD_GIL)
+#define setIsHoldGILDtor(cd) ((cd)->classflags |= CLASS_DTOR_HOLD_GIL)
+#define assignmentHelper(cd) ((cd)->classflags & CLASS_ASSIGN_HELPER)
+#define setAssignmentHelper(cd) ((cd)->classflags |= CLASS_ASSIGN_HELPER)
+#define noPyQt4QMetaObject(cd)  ((cd)->classflags & CLASS_NO_QMETAOBJECT)
+#define setPyQt4NoQMetaObject(cd)   ((cd)->classflags |= CLASS_NO_QMETAOBJECT)
+#define isTemplateClass(cd) ((cd)->classflags & CLASS_IS_TEMPLATE)
+#define setIsTemplateClass(cd)  ((cd)->classflags |= CLASS_IS_TEMPLATE)
+#define resetIsTemplateClass(cd)    ((cd)->classflags &= ~CLASS_IS_TEMPLATE)
+#define isDeprecatedClass(cd)   ((cd)->classflags & CLASS_IS_DEPRECATED)
+#define setIsDeprecatedClass(cd)    ((cd)->classflags |= CLASS_IS_DEPRECATED)
+#define cannotCopy(cd)      ((cd)->classflags & CLASS_CANNOT_COPY)
+#define setCannotCopy(cd)   ((cd)->classflags |= CLASS_CANNOT_COPY)
+#define cannotAssign(cd)    ((cd)->classflags & CLASS_CANNOT_ASSIGN)
+#define setCannotAssign(cd) ((cd)->classflags |= CLASS_CANNOT_ASSIGN)
+#define classHandlesNone(cd)    ((cd)->classflags & CLASS_ALLOW_NONE)
+#define setClassHandlesNone(cd) ((cd)->classflags |= CLASS_ALLOW_NONE)
 
 #define isPublicDtor(cd)    ((cd)->classflags & SECT_IS_PUBLIC)
 #define setIsPublicDtor(cd) ((cd)->classflags |= SECT_IS_PUBLIC)
@@ -148,6 +165,8 @@
 #define CTOR_CAST           0x00000400  /* The ctor is a cast. */
 #define CTOR_HOLD_GIL       0x00000800  /* The ctor holds the GIL. */
 #define CTOR_XFERRED        0x00001000  /* Ownership is transferred. */
+#define CTOR_IS_DEPRECATED  0x00002000  /* The ctor is deprecated. */
+#define CTOR_KEYWORD_ARGS   0x00004000  /* The ctor allows keyword arguments. */
 
 #define isPublicCtor(c)     ((c)->ctorflags & SECT_IS_PUBLIC)
 #define setIsPublicCtor(c)  ((c)->ctorflags |= SECT_IS_PUBLIC)
@@ -165,23 +184,32 @@
 #define setIsHoldGILCtor(c) ((c)->ctorflags |= CTOR_HOLD_GIL)
 #define isResultTransferredCtor(c)  ((c)->ctorflags & CTOR_XFERRED)
 #define setIsResultTransferredCtor(c)   ((c)->ctorflags |= CTOR_XFERRED)
+#define isDeprecatedCtor(c) ((c)->ctorflags & CTOR_IS_DEPRECATED)
+#define setIsDeprecatedCtor(c)  ((c)->ctorflags |= CTOR_IS_DEPRECATED)
+#define useKeywordArgsCtor(c)   ((c)->ctorflags & CTOR_KEYWORD_ARGS)
+#define setUseKeywordArgsCtor(c)    ((c)->ctorflags |= CTOR_KEYWORD_ARGS)
 
 
 /* Handle member flags. */
 
 #define MEMBR_NUMERIC       0x0001      /* It is a numeric slot. */
 #define MEMBR_NO_ARG_PARSER 0x0002      /* Don't generate an argument parser. */
+#define MEMBR_NOT_VERSIONED 0x0004      /* There is an unversioned overload. */
+#define MEMBR_KEYWORD_ARGS  0x0008      /* It allows keyword arguments. */
 
 #define isNumeric(m)        ((m)->memberflags & MEMBR_NUMERIC)
 #define setIsNumeric(m)     ((m)->memberflags |= MEMBR_NUMERIC)
 #define noArgParser(m)      ((m)->memberflags & MEMBR_NO_ARG_PARSER)
 #define setNoArgParser(m)   ((m)->memberflags |= MEMBR_NO_ARG_PARSER)
+#define notVersioned(m)     ((m)->memberflags & MEMBR_NOT_VERSIONED)
+#define setNotVersioned(m)  ((m)->memberflags |= MEMBR_NOT_VERSIONED)
+#define useKeywordArgsFunction(m)   ((m)->memberflags & MEMBR_KEYWORD_ARGS)
+#define setUseKeywordArgsFunction(m)    ((m)->memberflags |= MEMBR_KEYWORD_ARGS)
 
 
 /* Handle enum flags.  These are combined with the section flags. */
 
 #define ENUM_WAS_PROT       0x00000100  /* It was defined as protected. */
-#define ENUM_IS_RENAMED     0x00000200  /* It has been renamed. */
 
 #define isProtectedEnum(e)  ((e)->enumflags & SECT_IS_PROT)
 #define setIsProtectedEnum(e)   ((e)->enumflags |= SECT_IS_PROT)
@@ -190,19 +218,21 @@
 #define wasProtectedEnum(e) ((e)->enumflags & ENUM_WAS_PROT)
 #define setWasProtectedEnum(e)  ((e)->enumflags |= ENUM_WAS_PROT)
 #define resetWasProtectedEnum(e)    ((e)->enumflags &= ~ENUM_WAS_PROT)
-#define isRenamedEnum(e)    ((e)->enumflags & ENUM_IS_RENAMED)
-#define setIsRenamedEnum(e) ((e)->enumflags |= ENUM_IS_RENAMED)
 
 
 /* Handle hierarchy flags. */
 
 #define HIER_IS_DUPLICATE   0x0001      /* It is a super class duplicate. */
 #define HIER_HAS_DUPLICATE  0x0002      /* It has a super class duplicate. */
+#define HIER_BEING_SET      0x0004      /* The MRO is being set. */
 
 #define isDuplicateSuper(m) ((m)->mroflags & HIER_IS_DUPLICATE)
 #define setIsDuplicateSuper(m)  ((m)->mroflags |= HIER_IS_DUPLICATE)
 #define hasDuplicateSuper(m)    ((m)->mroflags & HIER_HAS_DUPLICATE)
 #define setHasDuplicateSuper(m) ((m)->mroflags |= HIER_HAS_DUPLICATE)
+#define hierBeingSet(m)     ((m)->mroflags & HIER_BEING_SET)
+#define setHierBeingSet(m)  ((m)->mroflags |= HIER_BEING_SET)
+#define resetHierBeingSet(m)    ((m)->mroflags &= ~HIER_BEING_SET)
 
 
 /* Handle overload flags.  These are combined with the section flags. */
@@ -223,6 +253,9 @@
 #define OVER_THIS_XFERRED   0x00200000  /* Ownership of this is transferred. */
 #define OVER_IS_GLOBAL      0x00400000  /* It is a global operator. */
 #define OVER_IS_COMPLEMENTARY   0x00800000  /* It is a complementary operator. */
+#define OVER_IS_DEPRECATED  0x01000000  /* It is deprecated. */
+#define OVER_KEYWORD_ARGS   0x02000000  /* It allows keyword arguments. */
+#define OVER_REALLY_PROT    0x04000000  /* It really is protected. */
 
 #define isPublic(o)         ((o)->overflags & SECT_IS_PUBLIC)
 #define setIsPublic(o)      ((o)->overflags |= SECT_IS_PUBLIC)
@@ -232,8 +265,10 @@
 #define setIsPrivate(o)     ((o)->overflags |= SECT_IS_PRIVATE)
 #define isSlot(o)           ((o)->overflags & SECT_IS_SLOT)
 #define setIsSlot(o)        ((o)->overflags |= SECT_IS_SLOT)
+#define resetIsSlot(o)      ((o)->overflags &= ~SECT_IS_SLOT)
 #define isSignal(o)         ((o)->overflags & SECT_IS_SIGNAL)
 #define setIsSignal(o)      ((o)->overflags |= SECT_IS_SIGNAL)
+#define resetIsSignal(o)    ((o)->overflags &= ~SECT_IS_SIGNAL)
 
 #define isVirtual(o)        ((o)->overflags & OVER_IS_VIRTUAL)
 #define setIsVirtual(o)     ((o)->overflags |= OVER_IS_VIRTUAL)
@@ -269,6 +304,12 @@
 #define setIsGlobal(o)      ((o)->overflags |= OVER_IS_GLOBAL)
 #define isComplementary(o)  ((o)->overflags & OVER_IS_COMPLEMENTARY)
 #define setIsComplementary(o)   ((o)->overflags |= OVER_IS_COMPLEMENTARY)
+#define isDeprecated(o)     ((o)->overflags & OVER_IS_DEPRECATED)
+#define setIsDeprecated(o)  ((o)->overflags |= OVER_IS_DEPRECATED)
+#define useKeywordArgs(o)   ((o)->overflags & OVER_KEYWORD_ARGS)
+#define setUseKeywordArgs(o)    ((o)->overflags |= OVER_KEYWORD_ARGS)
+#define isReallyProtected(o)    ((o)->overflags & OVER_REALLY_PROT)
+#define setIsReallyProtected(o) ((o)->overflags |= OVER_REALLY_PROT)
 
 
 /* Handle variable flags. */
@@ -298,9 +339,11 @@
 #define ARG_CONSTRAINED     0x0800  /* Suppress type conversion. */
 #define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
 #define ARG_RESULT_SIZE     0x2000  /* It defines the result size. */
+#define ARG_KEEP_REF        0x4000  /* Keep a reference. */
+#define ARG_NO_COPY         0x8000  /* Disable copying of const references. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
-#define setIsReference(a)   ((a)-> argflags |= ARG_IS_REF)
+#define setIsReference(a)   ((a)->argflags |= ARG_IS_REF)
 #define resetIsReference(a) ((a)->argflags &= ~ARG_IS_REF)
 #define isConstArg(a)       ((a)->argflags & ARG_IS_CONST)
 #define setIsConstArg(a)    ((a)->argflags |= ARG_IS_CONST)
@@ -329,19 +372,22 @@
 #define isSingleShot(a)     ((a)->argflags & ARG_SINGLE_SHOT)
 #define isResultSize(a)     ((a)->argflags & ARG_RESULT_SIZE)
 #define setResultSize(a)    ((a)->argflags |= ARG_RESULT_SIZE)
+#define keepReference(a)    ((a)->argflags & ARG_KEEP_REF)
+#define setKeepReference(a) ((a)->argflags |= ARG_KEEP_REF)
+#define noCopy(a)           ((a)->argflags & ARG_NO_COPY)
+#define setNoCopy(a)        ((a)->argflags |= ARG_NO_COPY)
 
 
 /* Handle name flags. */
 
 #define NAME_IS_USED        0x01    /* It is used in the main module. */
-#define NAME_IS_CLASS       0x02    /* It is the name of a class. */
+#define NAME_IS_SUBSTR      0x02    /* It is a substring of another. */
 
 #define isUsedName(n)       ((n)->nameflags & NAME_IS_USED)
 #define setIsUsedName(n)    ((n)->nameflags |= NAME_IS_USED)
 #define resetIsUsedName(n)  ((n)->nameflags &= ~NAME_IS_USED)
-#define isClassName(n)      ((n)->nameflags & NAME_IS_CLASS)
-#define setIsClassName(n)   ((n)->nameflags |= NAME_IS_CLASS)
-#define resetIsClassName(n) ((n)->nameflags &= ~NAME_IS_CLASS)
+#define isSubstring(n)      ((n)->nameflags & NAME_IS_SUBSTR)
+#define setIsSubstring(n)   ((n)->nameflags |= NAME_IS_SUBSTR)
 
 
 /* Handle virtual handler flags. */
@@ -359,9 +405,20 @@
 /* Handle mapped type flags. */
 
 #define MT_NO_RELEASE       0x01    /* Do not generate a release function. */
+#define MT_ALLOW_NONE       0x02    /* The mapped type will handle None. */
 
 #define noRelease(mt)       ((mt)->mtflags & MT_NO_RELEASE)
 #define setNoRelease(mt)    ((mt)->mtflags |= MT_NO_RELEASE)
+#define handlesNone(mt)     ((mt)->mtflags & MT_ALLOW_NONE)
+#define setHandlesNone(mt)  ((mt)->mtflags |= MT_ALLOW_NONE)
+
+
+/* Handle typedef flags. */
+
+#define TD_NO_TYPE_NAME     0x01    /* Do not use the typedef name. */
+
+#define noTypeName(td)      ((td)->tdflags & TD_NO_TYPE_NAME)
+#define setNoTypeName(td)   ((td)->tdflags |= TD_NO_TYPE_NAME)
 
 
 /* Slot types. */
@@ -381,6 +438,8 @@ typedef enum {
     repeat_slot,
     div_slot,
     mod_slot,
+    floordiv_slot,
+    truediv_slot,
     and_slot,
     or_slot,
     xor_slot,
@@ -393,6 +452,8 @@ typedef enum {
     irepeat_slot,
     idiv_slot,
     imod_slot,
+    ifloordiv_slot,
+    itruediv_slot,
     iand_slot,
     ior_slot,
     ixor_slot,
@@ -410,12 +471,15 @@ typedef enum {
     gt_slot,
     ge_slot,
     cmp_slot,
-    nonzero_slot,
+    bool_slot,
     neg_slot,
     pos_slot,
     abs_slot,
     repr_slot,
     hash_slot,
+    index_slot,
+    iter_slot,
+    next_slot,
     no_slot
 } slotType;
 
@@ -470,7 +534,10 @@ typedef enum {
     sstring_type,
     wstring_type,
     fake_void_type,
-    ssize_type
+    ssize_type,
+    ascii_string_type,
+    latin1_string_type,
+    utf8_string_type
 } argType;
 
 
@@ -539,26 +606,13 @@ typedef struct _scopedNameDef {
 
 typedef struct _nameDef {
     int nameflags;                      /* The name flags. */
-    struct _moduleDef *module;          /* The main module. */
     const char *text;                   /* The text of the name. */
+    size_t len;                         /* The length of the name. */
+    size_t offset;                      /* The offset in the string pool. */
     struct _nameDef *next;              /* Next in the list. */
 } nameDef;
 
 
-/*
- * A node in the tree of classes used to determine the order in which the
- * classes need to be created.
- */
-
-typedef struct _nodeDef {
-    int ordered;                        /* Set if in order. */
-    struct _classDef *cd;               /* The class. */
-    struct _nodeDef *parent;            /* The parent. */
-    struct _nodeDef *child;             /* The first child. */
-    struct _nodeDef *next;              /* The next sibling. */
-} nodeDef;
-
-
 /* A literal code block. */
 
 typedef struct _codeBlock {
@@ -569,52 +623,6 @@ typedef struct _codeBlock {
 } codeBlock;
 
 
-/* A module definition. */
-
-typedef struct _moduleDef {
-    const char *fullname;               /* The full module name. */
-    const char *name;                   /* The module base name. */
-    int version;                        /* The module version. */
-    int modflags;                       /* The module flags. */
-    int qobjclass;                      /* QObject class, -1 if none. */
-    struct _memberDef *othfuncs;        /* List of other functions. */
-    struct _overDef *overs;             /* Global overloads. */
-    codeBlock *hdrcode;                 /* Header code. */
-    codeBlock *cppcode;                 /* Global C++ code. */
-    codeBlock *copying;                 /* Software license. */
-    codeBlock *preinitcode;             /* Pre-initialisation code. */
-    codeBlock *postinitcode;            /* Post-initialisation code. */
-    codeBlock *unitcode;                /* Compilation unit code. */
-    int parts;                          /* The number of parts generated. */
-    char *file;                         /* The filename. */
-    qualDef *qualifiers;                /* The list of qualifiers. */
-    nodeDef root;                       /* Root of class tree. */
-    int nrtimelines;                    /* The nr. of timelines. */
-    int nrclasses;                      /* The nr. of classes. */
-    int nrexceptions;                   /* The nr. of exceptions. */
-    int nrmappedtypes;                  /* The nr. of mapped types. */
-    int nrenums;                        /* The nr. of named enums. */
-    int nrtypedefs;                     /* The nr. of typedefs. */
-    int nrvirthandlers;                 /* The nr. of virtual handlers. */
-    struct _virtHandlerDef *virthandlers;   /* The virtual handlers. */
-    licenseDef *license;                /* The software license. */
-    struct _classDef *proxies;          /* The list of proxy classes. */
-    struct _moduleDef *container;       /* The container module, if any. */
-    struct _ifaceFileList *used;        /* Interface files used. */
-    struct _moduleListDef *allimports;  /* The list of all imports. */
-    struct _moduleListDef *imports;     /* The list of direct imports. */
-    struct _moduleDef *next;            /* Next in the list. */
-} moduleDef;
-
-
-/* An entry in a linked module list. */
-
-typedef struct _moduleListDef {
-    moduleDef *module;                  /* The module itself. */
-    struct _moduleListDef *next;        /* The next in the list. */
-} moduleListDef;
-
-
 /* The arguments to a throw specifier. */
 
 typedef struct _throwArgs {
@@ -628,7 +636,7 @@ typedef struct _throwArgs {
 typedef struct _exceptionDef {
     int exceptionnr;                    /* The exception number. */
     struct _ifaceFileDef *iff;          /* The interface file. */
-    char *pyname;                       /* The exception Python name. */
+    const char *pyname;                 /* The exception Python name. */
     struct _classDef *cd;               /* The exception class. */
     char *bibase;                       /* The builtin base exception. */
     struct _exceptionDef *base;         /* The defined base exception. */
@@ -659,10 +667,14 @@ typedef struct _valueDef {
 
 typedef struct {
     argType atype;                      /* The type. */
-    char *name;                         /* The name. */
+    nameDef *name;                      /* The name. */
+    const char *doctype;                /* The documented type. */
     int argflags;                       /* The argument flags. */
     int nrderefs;                       /* Nr. of dereferences. */
     valueDef *defval;                   /* The default value. */
+    const char *docval;                 /* The documented value. */
+    int key;                            /* The optional /KeepReference/ key. */
+    struct _typedefDef *original_type;  /* The original type if typedef'd. */
     union {
         struct _signatureDef *sa;       /* If it is a function. */
         struct _templateDef *td;        /* If it is a template. */
@@ -692,11 +704,75 @@ typedef struct _fcallDef {
 } fcallDef;
 
 
+/* An API version range definition. */
+typedef struct _apiVersionRangeDef {
+    nameDef *api_name;                  /* The API name. */
+    int from;                           /* The lower bound. */
+    int to;                             /* The upper bound. */
+    int index;                          /* The range index. */
+    struct _apiVersionRangeDef *next;   /* The next in the list. */
+} apiVersionRangeDef;
+
+
+/* A module definition. */
+typedef struct _moduleDef {
+    nameDef *fullname;                  /* The full module name. */
+    const char *name;                   /* The module base name. */
+    int version;                        /* The module version. */
+    apiVersionRangeDef *api_versions;   /* The defined APIs. */
+    apiVersionRangeDef *api_ranges;     /* The list of API version ranges. */
+    int modflags;                       /* The module flags. */
+    int qobjclass;                      /* QObject class, -1 if none. */
+    struct _memberDef *othfuncs;        /* List of other functions. */
+    struct _overDef *overs;             /* Global overloads. */
+    argType encoding;                   /* The default string encoding. */
+    nameDef *defmetatype;               /* The optional default meta-type. */
+    nameDef *defsupertype;              /* The optional default super-type. */
+    struct _exceptionDef *defexception; /* The default exception. */
+    codeBlock *hdrcode;                 /* Header code. */
+    codeBlock *cppcode;                 /* Global C++ code. */
+    codeBlock *copying;                 /* Software license. */
+    codeBlock *preinitcode;             /* Pre-initialisation code. */
+    codeBlock *initcode;                /* Initialisation code. */
+    codeBlock *postinitcode;            /* Post-initialisation code. */
+    codeBlock *unitcode;                /* Compilation unit code. */
+    int parts;                          /* The number of parts generated. */
+    char *file;                         /* The filename. */
+    qualDef *qualifiers;                /* The list of qualifiers. */
+    argDef *types;                      /* The array of numbered types. */
+    int nrtypes;                        /* The number of numbered types. */
+    int nrtimelines;                    /* The nr. of timelines. */
+    int nrexceptions;                   /* The nr. of exceptions. */
+    int nrtypedefs;                     /* The nr. of typedefs. */
+    int nrvirthandlers;                 /* The nr. of virtual handlers. */
+    int next_key;                       /* The next key to allocate. */
+    struct _virtHandlerDef *virthandlers;   /* The virtual handlers. */
+    licenseDef *license;                /* The software license. */
+    struct _classDef *proxies;          /* The list of proxy classes. */
+    struct _moduleDef *container;       /* The container module, if any. */
+    struct _ifaceFileList *used;        /* Interface files used. */
+    struct _moduleListDef *allimports;  /* The list of all imports. */
+    struct _moduleListDef *imports;     /* The list of direct imports. */
+    struct _moduleDef *next;            /* Next in the list. */
+} moduleDef;
+
+
+/* An entry in a linked module list. */
+typedef struct _moduleListDef {
+    moduleDef *module;                  /* The module itself. */
+    struct _moduleListDef *next;        /* The next in the list. */
+} moduleListDef;
+
+
 /* An interface file definition. */
 
 typedef struct _ifaceFileDef {
     nameDef *name;                      /* The name. */
+    apiVersionRangeDef *api_range;      /* The optional API version range. */
+    struct _ifaceFileDef *first_alt;    /* The first alternate API. */
+    struct _ifaceFileDef *next_alt;     /* The next alternate API. */
     ifaceFileType type;                 /* Interface file type. */
+    int ifacenr;                        /* The index into the types table. */
     scopedNameDef *fqcname;             /* The fully qualified C++ name. */
     moduleDef *module;                  /* The owning module. */
     codeBlock *hdrcode;                 /* Header code. */
@@ -718,8 +794,12 @@ typedef struct _ifaceFileList {
 typedef struct _mappedTypeDef {
     int mtflags;                        /* The mapped type flags. */
     argDef type;                        /* The type being mapped. */
-    int mappednr;                       /* The mapped type number. */
+    nameDef *pyname;                    /* The Python name. */
+    nameDef *cname;                     /* The C/C++ name. */
+    const char *doctype;                /* The documented type. */
     ifaceFileDef *iff;                  /* The interface file. */
+    struct _memberDef *members;         /* The static member functions. */
+    struct _overDef *overs;             /* The static overloads. */
     codeBlock *convfromcode;            /* Convert from C++ code. */
     codeBlock *convtocode;              /* Convert to C++ code. */
     struct _mappedTypeDef *next;        /* Next in the list. */
@@ -767,6 +847,7 @@ typedef struct _virtHandlerDef {
 /* A typedef definition. */
 
 typedef struct _typedefDef {
+    int tdflags;                        /* The typedef flags. */
     scopedNameDef *fqname;              /* The fully qualified name. */
     struct _classDef *ecd;              /* The enclosing class. */
     moduleDef *module;                  /* The owning module. */
@@ -797,6 +878,7 @@ typedef struct _overDef {
     char *cppname;                      /* The C++ name. */
     int overflags;                      /* The overload flags. */
     struct _memberDef *common;          /* Common parts. */
+    apiVersionRangeDef *api_range;      /* The optional API version range. */
     signatureDef pysig;                 /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature. */
     throwArgs *exceptions;              /* The exceptions. */
@@ -812,6 +894,7 @@ typedef struct _overDef {
 
 typedef struct _ctorDef {
     int ctorflags;                      /* The ctor flags. */
+    apiVersionRangeDef *api_range;      /* The optional API version range. */
     signatureDef pysig;                 /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature, NULL if /NoDerived/. */
     throwArgs *exceptions;              /* The exceptions. */
@@ -836,10 +919,15 @@ typedef struct _enumMemberDef {
 
 typedef struct _enumDef {
     int enumflags;                      /* The enum flags. */
-    scopedNameDef *fqcname;             /* The name (may be NULL). */
     nameDef *pyname;                    /* The Python name (may be NULL). */
+    scopedNameDef *fqcname;             /* The C/C++ name (may be NULL). */
+    nameDef *cname;                     /* The C/C++ name (may be NULL). */
+    struct _enumDef *first_alt;         /* The first alternate API. */
+    struct _enumDef *next_alt;          /* The next alternate API. */
     int enumnr;                         /* The enum number. */
-    struct _classDef *ecd;              /* The enclosing class. */
+    int enum_idx;                       /* The enum index within the module. */
+    struct _classDef *ecd;              /* The enclosing class, if any. */
+    struct _mappedTypeDef *emtd;        /* The enclosing mapped type, if any. */
     moduleDef *module;                  /* The owning module. */
     enumMemberDef *members;             /* The list of members. */
     struct _memberDef *slots;           /* The list of slots. */
@@ -853,8 +941,10 @@ typedef struct _enumDef {
 typedef struct _memberDef {
     nameDef *pyname;                    /* The Python name. */
     int memberflags;                    /* The member flags. */
+    int membernr;                       /* The index in the method table. */
     slotType slot;                      /* The slot type. */
     moduleDef *module;                  /* The owning module. */
+    codeBlock *docstring;               /* The overloads docstrings. */
     struct _memberDef *next;            /* Next in the list. */
 } memberDef;
 
@@ -898,15 +988,15 @@ typedef struct _mroDef {
 
 typedef struct _classDef {
     int classflags;                     /* The class flags. */
-    int userflags;                      /* The user type flags. */
-    int classnr;                        /* The class number. */
-    const char *pyname;                 /* The Python name. */
+    int pyqt4_flags;                    /* The PyQt4 specific flags. */
+    nameDef *pyname;                    /* The Python name. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _classDef *ecd;              /* The enclosing scope. */
     struct _classDef *real;             /* The real class if this is a proxy or extender. */
-    nodeDef *node;                      /* Position in class tree. */
     classList *supers;                  /* The parent classes. */
     mroDef *mro;                        /* The super-class hierarchy. */
+    nameDef *metatype;                  /* The meta-type. */
+    nameDef *supertype;                 /* The super-type. */
     templateDef *td;                    /* The instantiated template. */
     ctorDef *ctors;                     /* The constructors. */
     ctorDef *defctor;                   /* The default ctor. */
@@ -921,13 +1011,16 @@ typedef struct _classDef {
     codeBlock *cppcode;                 /* Class C++ code. */
     codeBlock *convtosubcode;           /* Convert to sub C++ code. */
     struct _classDef *subbase;          /* Sub-class base class. */
+    codeBlock *docstring;               /* Ctor docstrings. */
     codeBlock *convtocode;              /* Convert to C++ code. */
     codeBlock *travcode;                /* Traverse code. */
     codeBlock *clearcode;               /* Clear code. */
-    codeBlock *readbufcode;             /* Read buffer code. */
-    codeBlock *writebufcode;            /* Write buffer code. */
-    codeBlock *segcountcode;            /* Segment count code. */
-    codeBlock *charbufcode;             /* Character buffer code. */
+    codeBlock *getbufcode;              /* Get buffer code (Python v3). */
+    codeBlock *releasebufcode;          /* Release buffer code (Python v3). */
+    codeBlock *readbufcode;             /* Read buffer code (Python v2). */
+    codeBlock *writebufcode;            /* Write buffer code (Python v2). */
+    codeBlock *segcountcode;            /* Segment count code (Python v2). */
+    codeBlock *charbufcode;             /* Character buffer code (Python v2). */
     codeBlock *picklecode;              /* Pickle code. */
     struct _classDef *next;             /* Next in the list. */
 } classDef;
@@ -970,7 +1063,7 @@ typedef struct {
     codeBlock *docs;                    /* Documentation. */
     int sigslots;                       /* Set if signals or slots are used. */
     int genc;                           /* Set if we are generating C code. */
-    struct _stringList *options;        /* The list of options. */
+    struct _stringList *plugins;        /* The list of plugins. */
 } sipSpec;
 
 
@@ -995,19 +1088,20 @@ extern char *sipVersion;                /* The version of SIP. */
 extern stringList *includeDirList;      /* The include directory list for SIP files. */
 
 
-void parse(sipSpec *,FILE *,char *,stringList *,stringList *);
+void parse(sipSpec *, FILE *, char *, stringList *, stringList *, int, int);
 void parserEOF(char *,parserContext *);
 void transform(sipSpec *);
 void generateCode(sipSpec *, char *, char *, char *, const char *, int, int,
-        int, int, stringList *, const char *);
+        int, int, stringList *, const char *, int);
 void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile);
 void generateXML(sipSpec *pt, moduleDef *mod, const char *xmlFile);
-void generateExpression(valueDef *vd, FILE *fp);
+void generateExpression(valueDef *vd, int in_str, FILE *fp);
 void warning(char *,...);
 void fatal(char *,...);
 void fatalScopedName(scopedNameDef *);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
-void *sipMalloc(size_t);
+void *sipMalloc(size_t n);
+void *sipCalloc(size_t nr, size_t n);
 char *sipStrdup(const char *);
 char *concat(const char *, ...);
 void append(char **, const char *);
@@ -1016,10 +1110,9 @@ int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
 int sameTemplateSignature(signatureDef *tmpl_sd, signatureDef *args_sd,
         int deep);
-int sameScopedName(scopedNameDef *,scopedNameDef *);
+int compareScopedNames(scopedNameDef *snd1, scopedNameDef *snd2);
 int sameBaseType(argDef *,argDef *);
 char *scopedNameTail(scopedNameDef *);
-scopedNameDef *text2scopePart(char *);
 scopedNameDef *copyScopedName(scopedNameDef *);
 void appendScopedName(scopedNameDef **,scopedNameDef *);
 void freeScopedName(scopedNameDef *);
@@ -1027,25 +1120,30 @@ void appendToClassList(classList **,classDef *);
 void appendCodeBlock(codeBlock **headp, codeBlock *new);
 void prcode(FILE *fp, const char *fmt, ...);
 void prOverloadName(FILE *fp, overDef *od);
+void prOverloadDecl(FILE *fp, ifaceFileDef *scope, overDef *od, int defval);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
-void prOverloadDecl(FILE *fp, classDef *context, overDef *od, int defval);
+int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
+        int names, int defaults, int in_str, int is_signal);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isIntReturnSlot(memberDef *md);
+int isSSizeReturnSlot(memberDef *md);
 int isLongReturnSlot(memberDef *md);
 int isVoidReturnSlot(memberDef *md);
 int isNumberSlot(memberDef *md);
 int isRichCompareSlot(memberDef *md);
-mappedTypeDef *allocMappedType(argDef *type);
+mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type);
 void appendString(stringList **headp, const char *s);
 void appendTypeStrings(scopedNameDef *ename, signatureDef *patt, signatureDef *src, signatureDef *known, scopedNameDef **names, scopedNameDef **values);
 codeBlock *templateCode(sipSpec *pt, ifaceFileList **used, codeBlock *ocb, scopedNameDef *names, scopedNameDef *values);
-ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod, scopedNameDef *fqname, ifaceFileType iftype, argDef *ad);
-int optNoEmitters(sipSpec *pt);
-int optRegisterTypes(sipSpec *pt);
-int optQ_OBJECT4(sipSpec *pt);
-int optAssignmentHelpers(sipSpec *pt);
+ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod,
+        scopedNameDef *fqname, ifaceFileType iftype,
+        apiVersionRangeDef *api_range, argDef *ad);
+int pluginPyQt3(sipSpec *pt);
+int pluginPyQt4(sipSpec *pt);
 void yywarning(char *);
 nameDef *cacheName(sipSpec *pt, const char *name);
+scopedNameDef *encodedTemplateName(templateDef *td);
+apiVersionRangeDef *findAPI(sipSpec *pt, const char *name);
 
 
 /* These are only here because bison publically references them. */
@@ -1059,7 +1157,9 @@ typedef enum {
     string_flag,
     name_flag,
     opt_name_flag,
-    integer_flag
+    dotted_name_flag,
+    integer_flag,
+    api_range_flag
 } flagType;
 
 typedef struct {
@@ -1068,6 +1168,7 @@ typedef struct {
     union {                             /* The flag value. */
         char *sval;                     /* A string value. */
         long ival;                      /* An integer value. */
+        apiVersionRangeDef *aval;       /* An API range value. */
     } fvalue;
 } optFlag;
 
