@@ -138,6 +138,19 @@ PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs)
     {
         PyObject *self = (sref != NULL ? sref : slot->meth.mself);
 
+        /*
+         * If the receiver wraps a C++ object then ignore the call if it no
+         * longer exists.
+         */
+        if (PyObject_TypeCheck(self, (PyTypeObject *)&sipSimpleWrapper_Type) &&
+            sipGetAddress(self) == NULL)
+        {
+            Py_XDECREF(sref);
+
+            Py_INCREF(Py_None);
+            return Py_None;
+        }
+
 #if PY_MAJOR_VERSION >= 3
         sfunc = PyMethod_New(slot->meth.mfunc, self);
 #else
