@@ -54,8 +54,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040a01
-#define SIP_VERSION_STR     "4.10.1"
+#define SIP_VERSION         0x040a05
+#define SIP_VERSION_STR     "4.10.5"
 
 
 /*
@@ -208,6 +208,10 @@ extern "C" {
 #define SIPBytes_AS_STRING  PyBytes_AS_STRING
 #define SIPBytes_GET_SIZE   PyBytes_GET_SIZE
 
+#if PY_MINOR_VERSION >= 1
+#define SIP_USE_PYCAPSULE
+#endif
+
 #else
 
 #define SIPLong_FromLong    PyInt_FromLong
@@ -219,6 +223,10 @@ extern "C" {
 #define SIPBytes_AS_STRING  PyString_AS_STRING
 #define SIPBytes_GET_SIZE   PyString_GET_SIZE
 
+#if PY_MINOR_VERSION >= 7
+#define SIP_USE_PYCAPSULE
+#endif
+
 #endif
 
 #if !defined(Py_REFCNT)
